/* SPDX-License-Identifier: (GPL-2.0-only OR MIT) */
/*
 * Copyright (C) 2025 Amlogic, Inc. All rights reserved
 */
#ifndef _DT_BINDINGS_AMLOGIC_S7_POWER_H
#define _DT_BINDINGS_AMLOGIC_S7_POWER_H

#define PWRC_S7_DOS_HEVC_ID			0
#define PWRC_S7_DOS_VDEC_ID		        1
#define PWRC_S7_VPU_HDMI_ID		        2
#define PWRC_S7_USB_COMB_ID		        3
#define PWRC_S7_SD_EMMC_C_ID			4
#define PWRC_S7_GE2D_ID				5
#define PWRC_S7_SD_EMMC_A_ID			6
#define PWRC_S7_SD_EMMC_B_ID			7
#define PWRC_S7_ETH_ID				8
#define PWRC_S7_AUCPU_ID			9
#define PWRC_S7_AUDIO_ID			10

#endif
