/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2018 The Linux Foundation. All rights reserved.
 */

#ifndef _DT_BINDINGS_RESET_AOSS_SDM_845_H
#define _DT_BINDINGS_RESET_AOSS_SDM_845_H

#define AOSS_CC_MSS_RESTART	0
#define AOSS_CC_CAMSS_RESTART	1
#define AOSS_CC_VENUS_RESTART	2
#define AOSS_CC_GPU_RESTART	3
#define AOSS_CC_DISPSS_RESTART	4
#define AOSS_CC_WCSS_RESTART	5
#define AOSS_CC_LPASS_RESTART	6

#endif
