savedcmd_arch/x86/include/generated/uapi/asm/unistd_64.h := sh ./scripts/syscallhdr.sh --abis common,64 --emit-nr   arch/x86/entry/syscalls/syscall_64.tbl arch/x86/include/generated/uapi/asm/unistd_64.h
