/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_CLK_MT6735_MFGCFG_H
#define _DT_BINDINGS_CLK_MT6735_MFGCFG_H

#define CLK_MFG_BG3D			0

#endif /* _DT_BINDINGS_CLK_MT6735_MFGCFG_H */
