/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (c) 2014, The Linux Foundation. All rights reserved.
 */

#ifndef _DT_BINDINGS_CLK_LCC_IPQ806X_H
#define _DT_BINDINGS_CLK_LCC_IPQ806X_H

#define PLL4				0
#define MI2S_OSR_SRC			1
#define MI2S_OSR_CLK			2
#define MI2S_DIV_CLK			3
#define MI2S_BIT_DIV_CLK		4
#define MI2S_BIT_CLK			5
#define PCM_SRC				6
#define PCM_CLK_OUT			7
#define PCM_CLK				8
#define SPDIF_SRC			9
#define SPDIF_CLK			10
#define AHBIX_CLK			11

#define LCC_PCM_RESET			0

#endif
