/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2020, The Linux Foundation. All rights reserved.
 * Copyright (c) 2020, AngeloGioacchino Del Regno <angelogioacchino.delregno@somainline.org>
 */

#ifndef _DT_BINDINGS_CLK_SDM_GPUCC_660_H
#define _DT_BINDINGS_CLK_SDM_GPUCC_660_H

#define GPUCC_CXO_CLK			0
#define GPU_PLL0_PLL			1
#define GPU_PLL1_PLL			2
#define GFX3D_CLK_SRC			3
#define RBCPR_CLK_SRC			4
#define RBBMTIMER_CLK_SRC		5
#define GPUCC_RBCPR_CLK			6
#define GPUCC_GFX3D_CLK			7
#define GPUCC_RBBMTIMER_CLK		8

#define GPU_CX_GDSC			0
#define GPU_GX_GDSC			1

#define GPU_CX_BCR			0
#define GPU_GX_BCR			1
#define RBCPR_BCR			2
#define SPDM_BCR			3

#endif
